from tkinter import *
from tkinter.ttk	import *
import time
import os
import sys
import glob
import Robot as rbt
from enum import Enum
from tkinter import font

caseXStart = 7
caseYStart = 2
caseXFin = 16
caseYFin = 11

class Mode(Enum):
    CIRCUIT = 1
    DEPLACEMENT = 2

class BoutonMode:
    def __init__(self, zoneDessin, x, y, tailleX, tailleY, texte, couleur = 'lightgray'):
        self.zD = zoneDessin
        self.couleur = couleur
        self.posX = x
        self.posY = y
        self.tailleX = tailleX
        self.tailleY = tailleY
        self.texte = texte
        self.rectangle = zoneDessin.create_rectangle(x, y, x+tailleX, y+tailleY, width=1, fill=self.couleur)
        self.texteRect = zoneDessin.create_text(x+tailleX/2, y+tailleY/2, text=texte)
    def getX(self):
        return self.x
    def getY(self):
        return self.y
    def isClicked(self, newX, newY):
        if((newX >= self.posX) and (newX <= self.posX + self.tailleX) and (newY >= self.posY) and (newY <= self.posY + self.tailleY)):
            return 1
        else:
            return 0
    def activeMode(self):
        self.zD.itemconfig(self.rectangle, fill='lightgreen')
    def desactiveMode(self):
        self.zD.itemconfig(self.rectangle, fill=self.couleur)

class LabelTexte:
    def __init__(self, zoneDessin, x, y, texte, taille = 20, variable = ''):
        self.zD = zoneDessin
        self.posX = x
        self.posY = y
        self.texte = texte
        self.taille = taille
        self.variable = variable
        self.fontL = font.Font(family='Helvetica', size=taille)
        self.labelL = zoneDessin.create_text(x, y, text=texte+variable, font = self.fontL)
    def changeText(self, texte):
        self.texte = texte
        self.zD.itemconfig(self.labelL, text=self.texte+self.variable)
    def changeVar(self, variable):
        self.variable = variable
        self.zD.itemconfig(self.labelL, text=self.texte+variable)

class Interface:
    "Interface TkInter pour jeu d'apprentissage de la programmation - LEnsE / VILLEMEJANE (c) 2020"

    def __init__(self):
        self.mode = Mode.CIRCUIT
        self.largeur = 17  #nombre de cases en largeur
        self.hauteur = 14  #nombre de bases en hauteur (surface de jeu de 10 x 10)
        self.nbPixelCase = 50   # taille d'une case de la zone d'affichage
        self.listeCases = []    # ensemble des cases de la zone d'affichage

        # Main window
        self.mainWindow = Tk()
        self.mainWindow.title("Apprendre la programmation / LEnsE / VILLEMEJANE")
        self.mainWindow.bind("<Key>", self.clavier)     # keyboard events attachement
        self.mainWindow.bind("<Button-1>", self.detectSouris)

        self.zoneDessin = Canvas(self.mainWindow, width = self.nbPixelCase * self.largeur, height = self.nbPixelCase * self.hauteur, bg ='white')
        self.zoneDessin.pack()
        # Robot
        self.robot = rbt.Robot(1, 1)
        self.robotCan = Canvas(self.mainWindow,height=10,width=10,bg='white')
        #self.robotCan.pack()
        self.robotRect = self.robotCan.create_rectangle(0,0,10,10,fill='red')

        # Affichage zone robot
        for i in range (self.hauteur) :
            for j in range(self.largeur) :
                if((i < caseYStart-1) or (i > caseYFin-1)) or ((j < caseXStart-1) or (j > caseXFin-1)):
                    self.zoneDessin.create_rectangle(j*self.nbPixelCase, i*self.nbPixelCase, j*self.nbPixelCase + self.nbPixelCase, i*self.nbPixelCase + self.nbPixelCase, width=0, fill='white')
                else:
                    rect = self.zoneDessin.create_rectangle(j*self.nbPixelCase, i*self.nbPixelCase, j*self.nbPixelCase + self.nbPixelCase, i*self.nbPixelCase + self.nbPixelCase, width=2, fill='lightgray')
                    self.listeCases.append(rect)

        # Affichage coord des cases
        for k in range (10) :
            self.zoneDessin.create_text((k+caseXStart-1)*self.nbPixelCase+25, (caseYStart-2)*self.nbPixelCase+25, text=str(k+1))
        for k in range (10) :
            self.zoneDessin.create_text((caseXStart-2)*self.nbPixelCase+25, (k+caseYStart-1)*self.nbPixelCase+25, text=str(k+1))
        # Affichage mode de jeu
        self.boutonModeDep = BoutonMode(self.zoneDessin,30,10,80,30,'Deplacement')
        self.boutonModeCircuit = BoutonMode(self.zoneDessin,130,10,80,30,'Circuit')
        self.boutonModeCircuit.activeMode()
        # Affichage controles robot
        self.boutonCtrlDroit = BoutonMode(self.zoneDessin,3*self.nbPixelCase + 5,3*self.nbPixelCase + 5,40,40,'D', 'blue')
        self.boutonCtrlGauche = BoutonMode(self.zoneDessin,1*self.nbPixelCase + 5,3*self.nbPixelCase + 5,40,40,'G', 'green')
        self.boutonCtrlAvance = BoutonMode(self.zoneDessin,2*self.nbPixelCase + 5,3*self.nbPixelCase + 5,40,40,'A', 'orange')
        # Affichage des paramètres du robot
        self.posXrbt = LabelTexte(self.zoneDessin, 120, 250, 'X = ',15, str(self.robot.getPosX()))
        self.posYrbt = LabelTexte(self.zoneDessin, 120, 300, 'Y = ',15, str(self.robot.getPosY()))

        self.miseAJourAffichage()

    def detectSouris(self, event):
        print('OK - X = '+str(event.x)+ ' / Y = ' + str(event.y))
        if(self.boutonModeCircuit.isClicked(event.x, event.y) == 1):
            self.mode = Mode.CIRCUIT
            self.boutonModeCircuit.activeMode()
            self.boutonModeDep.desactiveMode()
            print('MODE Circuit OK')
        elif(self.boutonModeDep.isClicked(event.x, event.y) == 1):
            self.mode = Mode.DEPLACEMENT
            self.boutonModeDep.activeMode()
            self.boutonModeCircuit.desactiveMode()
            print('MODE Deplacement OK')
        elif(self.boutonCtrlDroit.isClicked(event.x, event.y) == 1):
            self.tournerDroite()
            self.miseAJourAffichage()
        elif(self.boutonCtrlGauche.isClicked(event.x, event.y) == 1):
            self.tournerGauche()
            self.miseAJourAffichage()
        elif(self.boutonCtrlAvance.isClicked(event.x, event.y) == 1):
            self.avancer()
            self.miseAJourAffichage()

    def clavier(self, event):
        touche = event.keysym
        if touche == 'q' or touche == 'Q' :
            print('Exit')
            self.go = -1
        elif touche == 'g' or touche == 'G' :
            print('GO !')
            self.go = 1
        elif touche == 's' or touche == 'S' :
            print('STOP !!')
            self.go = 0

    def miseAJourAffichage(self):
        self.posXrbt.changeVar(str(self.robot.getPosX()))
        self.posYrbt.changeVar(str(self.robot.getPosY()))
        oldxrbt = self.robot.getOldPosX()-1
        xrbt = self.robot.getPosX()-1
        oldyrbt = self.robot.getOldPosY()-1
        yrbt = self.robot.getPosY()-1
        self.zoneDessin.itemconfig(self.listeCases[oldyrbt*10+oldxrbt], fill='lightgray', width=2)
        self.zoneDessin.itemconfig(self.listeCases[yrbt*10+xrbt], fill='yellow', width=5)
#        if(self.robot.getDirection() == rbt.Direction.BAS):
        self.robotCan.coords(xrbt, yrbt, xrbt+10, yrbt+10)
#        elif(self.robot.getDirection() == rbt.Direction.HAUT):
#            self.zoneDessin.itemconfig(self.robotRect, x0=xrbt, y0=yrbt, x1=xrbt+10, y1=yrbt+10)
#        elif(self.robot.getDirection() == rbt.Direction.GAUCHE):
#            self.zoneDessin.itemconfig(self.robotRect, x0=xrbt, y0=yrbt, x1=xrbt+10, y1=yrbt+10)
#        elif(self.robot.getDirection() == rbt.Direction.DROITE):
#            self.zoneDessin.itemconfig(self.robotRect, x0=xrbt, y0=yrbt, x1=xrbt+10, y1=yrbt+10)

    def tournerDroite(self):
        if(self.mode == Mode.DEPLACEMENT):
            self.robot.tournerDroite()
            print('Tourne à droite')
        elif(self.mode == Mode.CIRCUIT):
            print('TO DO')


    def tournerGauche(self):
        if(self.mode == Mode.DEPLACEMENT):
            self.robot.tournerGauche()
            print('Tourne à gauche')
        elif(self.mode == Mode.CIRCUIT):
            print('TO DO')

    def avancer(self):
        if(self.mode == Mode.DEPLACEMENT):
            self.robot.afficher()
            if(self.robot.getDirection() == rbt.Direction.GAUCHE):
                if(self.robot.getPosX() > 1):
                    self.robot.avancer()
                    print('Avance Horizontale')
                else:
                    print('Erreur Av Hor')
            elif(self.robot.getDirection() == rbt.Direction.DROITE):
                if(self.robot.getPosX() < 10):
                    self.robot.avancer()
                    print('Avance Horizontale')
                else:
                    print('Erreur Av Hor')
            elif(self.robot.getDirection() == rbt.Direction.HAUT):
                if(self.robot.getPosY() > 1):
                    self.robot.avancer()
                    print('Avance Verticale')
                else:
                    print('Erreur Av Ver')
            elif(self.robot.getDirection() == rbt.Direction.BAS):
                if(self.robot.getPosY() < 10):
                    self.robot.avancer()
                    print('Avance Verticale')
                else:
                    print('Erreur Av Ver')
        elif(self.mode == Mode.CIRCUIT):
            print('TO DO')
    def getTaillePixel(self):
        return self.nbPixelCase
    def run(self):
        # MAIN PROGRAM
        self.mainWindow.focus_force()
        self.mainWindow.mainloop()